@@ -1,23 +1,10 @@
 /*
-   Copyright (c) 2017-2019 Volker Krause <vkrause@kde.org>
+   SPDX-FileCopyrightText: 2017-2019 Volker Krause <vkrause@kde.org>
 
-   This library is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Library General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or (at your
-   option) any later version.
-
-   This library is distributed in the hope that it will be useful, but WITHOUT
-   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
-   License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-function parseText(text) {
+function parseSncfPdfText(text) {
     var reservations = new Array();
     var bookingRef = text.match(/(?:DOSSIER VOYAGE|BOOKING FILE REFERENCE) : +([A-Z0-9]{6})/);
 
@@ -67,65 +54,212 @@ function parseText(text) {
     return reservations;
 }
 
+function parseInouiPdfText(page)
+{
+    var reservations = new Array();
+    var text = page.textInRect(0.0, 0.0, 0.5, 1.0);
+
+    var date = text.match(/(\d+ [^ ]+ \d{4})\n/)
+    if (!date)
+        return reservations;
+    var pos = date.index + date[0].length;
+    while (true) {
+        var dep = text.substr(pos).match(/(\d+h\d+) +(.*)\n/);
+        if (!dep)
+            break;
+        pos += dep.index + dep[0].length;
+
+        var res = JsonLd.newTrainReservation();
+        res.reservationFor.departureTime = JsonLd.toDateTime(date[1] + dep[1], "d MMMM yyyyhh'h'mm", "fr");
+        res.reservationFor.departureStation.name = dep[2];
+
+        var arr = text.substr(pos).match(/(\d+h\d+) +(.*)\n/);
+        if (!arr)
+            break;
+        var endPos = arr.index + arr[0].length;
+        res.reservationFor.arrivalTime = JsonLd.toDateTime(date[1] + arr[1], "d MMMM yyyyhh'h'mm", "fr");
+        res.reservationFor.arrivalStation.name = arr[2];
+
+        var detailsText = text.substr(pos, endPos - arr[0].length);
+        var train = detailsText.match(/^ *(.*?) *-/);
+        res.reservationFor.trainNumber = train[1];
+        var seat = detailsText.match(/Voiture *(\d+) *Place *(\d+)/);
+        if (seat) {
+            res.reservedTicket.ticketedSeat.seatSection = seat[1];
+            res.reservedTicket.ticketedSeat.seatNumber = seat[2];
+        }
+
+        reservations.push(res);
+        if (endPos == 0)
+            break;
+        pos += endPos;
+    }
+
+    return reservations;
+}
+
+// see https://community.kde.org/KDE_PIM/KItinerary/SNCF_Barcodes
+function parseSncfBarcode(barcode)
+{
+    var reservations = new Array();
+
+    var res1 = JsonLd.newTrainReservation();
+    res1.reservationNumber = barcode.substr(4, 6);
+    res1.underName.familyName = barcode.substr(72, 19);
+    res1.underName.givenName = barcode.substr(91, 19);
+    res1.reservationFor.departureStation.name = barcode.substr(33, 5);
+    res1.reservationFor.departureStation.identifier = "sncf:" + barcode.substr(33, 5);
+    res1.reservationFor.arrivalStation.name = barcode.substr(38, 5);
+    res1.reservationFor.arrivalStation.identifier = "sncf:" + barcode.substr(38, 5);
+    res1.reservationFor.departureDay = JsonLd.toDateTime(barcode.substr(48, 5), "dd/MM", "fr");
+    res1.reservationFor.trainNumber = barcode.substr(43, 5);
+    res1.reservedTicket.ticketToken = "aztecCode:" + barcode;
+    res1.reservedTicket.ticketedSeat.seatingType = barcode.substr(110, 1);
+    reservations.push(res1);
+
+    if (barcode.substr(115, 1) != '0') {
+        var res2 = JsonLd.clone(res1);
+        res2.reservationFor.departureStation.name = barcode.substr(116, 5);
+        res2.reservationFor.departureStation.identifier = "sncf:" + barcode.substr(116, 5);
+        res2.reservationFor.arrivalStation.name = barcode.substr(121, 5);
+        res2.reservationFor.arrivalStation.identifier = "sncf:" + barcode.substr(121, 5);
+        res2.reservationFor.trainNumber = barcode.substr(126, 5);
+        res2.reservedTicket.ticketedSeat.seatingType = barcode.substr(115, 1);
+        reservations.push(res2);
+    }
+
+    return reservations;
+}
+
 function parsePdf(pdf) {
     var reservations = new Array();
 
     var barcode = null;
     for (var i = 0; i < pdf.pageCount; ++i) {
         var page = pdf.pages[i];
-
-        // barcode format: (see https://community.kde.org/KDE_PIM/KItinerary/SNCF_Barcodes)
-        // 'i0CV'
-        // 6x PNR
-        // 9x document id / e-ticket number
-        // '1211'
-        // dd/MM/yyyy birthdate
-        // 2x 5x SNCF station code of the first leg
-        // 5x train number first leg
-        // dd/MM travel date
-        // 18x client id
-        // 19x family name
-        // 19x given name
-        // 1x class first leg ('1' or '2')
-        // 4x tariff/price code
-        // 1x class second leg ('1' or '2'; '0' if there is no second leg)
-        // 2x 5x SNCF station code for the second leg
-        // 5x train number second leg
         var nextBarcode = null;
-        var images = page.imagesInRect(0.75, 0, 1, 0.75);
+        var images = page.images;
         for (var j = 0; j < images.length && !nextBarcode; ++j) {
             nextBarcode = Barcode.decodeAztec(images[j]);
-            if (nextBarcode.substr(0, 4).toUpperCase() !== "I0CV")
+            if (nextBarcode.substr(0, 4) !== "i0CV")
                 nextBarcode = null;
         }
         // Guard against tickets with 3 or more legs, with the second page for the 3rd and subsequent
         // leg repeating the barcode of the first two legs. One would expect the barcode for the following
         // legs there, but that doesn't even seem to exists in the sample documents I have for this...
         barcode = (nextBarcode && nextBarcode != barcode) ? nextBarcode : null;
-
-        var underName = null;
         if (barcode) {
-            var underName = JsonLd.newObject("Person");
-            underName.familyName = barcode.substr(72, 19).trim();
-            underName.givenName = barcode.substr(91, 19).trim();
+            var barcodeRes = barcode ? parseSncfBarcode(barcode) : null;
         }
 
-        var legs = parseText(page.text);
-        for (var j = 0; j < legs.length; ++j) {
-            if (barcode) {
-                legs[j].underName = underName;
-                legs[j].reservedTicket.ticketToken = "aztecCode:" + barcode;
-                legs[j].reservationFor.departureStation.identifier = "sncf:" + barcode.substr(j == 0 ? 33 : 116, 5);
-                legs[j].reservationFor.arrivalStation.identifier = "sncf:" + barcode.substr(j == 0 ? 38 : 121, 5);
-                legs[j].reservedTicket.ticketedSeat.seatingType = barcode.substr(j == 0 ? 110 : 115, 1);
+        var legs = parseSncfPdfText(page.text);
+        if (legs.length == 0) {
+            legs = parseInouiPdfText(page);
+        }
+        if (legs.length > 0) {
+            for (var j = 0; j < legs.length; ++j) {
+                if (barcode) {
+                    legs[j].underName = barcodeRes[j].underName;
+                    legs[j].reservedTicket.ticketToken = "aztecCode:" + barcode;
+                    legs[j].reservationFor.departureStation.identifier = barcodeRes[j].reservationFor.departureStation.identifier;
+                    legs[j].reservationFor.arrivalStation.identifier = barcodeRes[j].reservationFor.arrivalStation.identifier;
+                    legs[j].reservedTicket.ticketedSeat.seatingType = barcodeRes[j].reservedTicket.ticketedSeat.seatingType;
+                }
+                reservations.push(legs[j]);
             }
-            reservations.push(legs[j]);
+        } else {
+            reservations = reservations.concat(barcodeRes);
         }
     }
 
     return reservations;
 }
 
+function parseSecutixPdf(pdf)
+{
+    // see https://community.kde.org/KDE_PIM/KItinerary/SNCF_Barcodes#SNCF_Secutix_Tickets
+    var res = JsonLd.newTrainReservation();
+    var code = Barcode.byteArrayToString(Context.barcode);
+    res.reservationNumber = code.substr(268, 9);
+    res.reservationFor.departureStation.name = code.substr(277, 5);
+    res.reservationFor.departureStation.identifier = "sncf:" + code.substr(277, 5);
+    res.reservationFor.arrivalStation.name = code.substr(282, 5);
+    res.reservationFor.arrivalStation.identifier = "sncf:" + code.substr(282, 5);
+    res.reservationFor.departureDay = JsonLd.toDateTime(code.substr(343, 8), "ddMMyyyy", "fr");
+    res.reservedTicket.ticketedSeat.seatingType = code.substr(351, 1);
+    res.reservedTicket.ticketToken = "aztecbin:" + Barcode.toBase64(Context.barcode);
+    res.underName.familyName = code.substr(376, 19);
+    res.underName.givenName = code.substr(395, 19);
+
+    var text = pdf.pages[Context.pdfPageNumber].text;
+    var pnr = text.match(res.reservationNumber + '[^\n]* ([A-Z0-9]{6})\n');
+    res.reservationNumber = pnr[1];
+
+    var reservations = new Array();
+    var pos = 0;
+    while (true) {
+        var dep = text.substr(pos).match(/Départ [^ ]+ (\d+\.\d+\.\d+) à (\d+:\d+) [^ ]+ (.*)\n/);
+        if (!dep)
+            break;
+        pos += dep.index + dep[0].length;
+        var arr = text.substr(pos).match(/Arrivée [^ ]+ (\d+\.\d+\.\d+) à (\d+:\d+) [^ ]+ (.*)\n\s+(.*)\n/);
+        if (!arr)
+            break;
+        pos += arr.index + arr[0].length;
+
+        var leg = JsonLd.newTrainReservation();
+        leg.reservationFor.departureStation.name = dep[3];
+        leg.reservationFor.departureTime = JsonLd.toDateTime(dep[1] + dep[2], "dd.MM.yyyyhh:mm", "fr");
+        leg.reservationFor.arrivalStation.name = arr[3];
+        leg.reservationFor.arrivalTime = JsonLd.toDateTime(arr[1] + arr[2], "dd.MM.yyyyhh:mm", "fr");
+        leg.reservationFor.trainNumber = arr[4];
+        leg.underName = res.underName;
+        leg.reservationNumber = res.reservationNumber;
+        leg.reservedTicket = res.reservedTicket;
+
+        reservations.push(leg);
+    }
+
+    if (reservations.length == 0)
+        return res;
+
+    reservations[0].reservationFor.departureStation.identifier = res.reservationFor.departureStation.identifier;
+    reservations[reservations.length - 1].reservationFor.arrivalStation.identifier = res.reservationFor.arrivalStation.identifier;
+
+    return reservations;
+}
+
+function parseOuigoEmail(html)
+{
+    if (html.eval('//*[@data-select="travel-summary-reference"]').length > 0) {
+        return parseOuigoSummary(html);
+    } else {
+        return parseOuigoConfirmation(html);
+    }
+}
+
+function parseOuigoSummary(html)
+{
+    // TODO extract passenger names
+    var res = JsonLd.newTrainReservation();
+    res.reservationFor.departureStation.name = html.eval('//*[@data-select="travel-summary-origin"]')[0].content;
+    res.reservationFor.arrivalStation.name = html.eval('//*[@data-select="travel-summary-destination"]')[0].content;
+    res.reservationNumber = html.eval('//*[@data-select="travel-summary-reference"]')[0].content;
+
+    var depTimeStr = html.eval('//*[@data-select="travel-departureDate"]')[0].recursiveContent;
+    var depTime = depTimeStr.match(/(\d+ [^ ]+ \d+) +[^ ]+ (\d+:\d+)/);
+    if (depTime) {
+        res.reservationFor.departureTime = JsonLd.toDateTime(depTime[1] + depTime[2], "dd MMMM yyyyhh:mm", "fr");
+    } else {
+        depTime = depTimeStr.match(/(\d+ [^ ]+) +[^ ]+ +(\d+[:h]\d+)/);
+        res.reservationFor.departureTime = JsonLd.toDateTime(depTime[1] + depTime[2].replace('h', ':'), "dd MMMMhh:mm", "fr");
+    }
+
+    var trainNum = html.eval('//*[@class="passenger-detail__equipment"]');
+    res.reservationFor.trainNumber = trainNum[0].content + " " + trainNum[1].content;
+    return res;
+}
+
 function parseOuigoConfirmation(html)
 {
     var reservations = new Array();
@@ -207,7 +341,7 @@ function parseOuigoTicket(pdf) {
     res.reservedTicket.ticketedSeat.seatSection = seat[1];
     res.reservedTicket.ticketedSeat.seatNumber = seat[2];
     if (Context.barcode) {
-        res.reservedTicket.ticketToken = "acteccode:" + Context.barcode;
+        res.reservedTicket.ticketToken = "azteccode:" + Context.barcode;
     }
     return res;
 }
